# $Id: StyleProvider.pm,v 1.3 2000/09/14 20:44:59 matt Exp $

package Apache::AxKit::StyleProvider;
use strict;

die "Do not use this module!";

1;
__END__

=head1 NAME

Apache::AxKit::StyleProvider - DEPRECATED!!!

=head1 DESCRIPTION

This module is deprecated. Please do not derive from it or use it.

=cut
