body {
	color:		 #333;
	margin:		 0px;
}

p {
	font-family: Arial,Helvetica,Helv,sans-serif;
}


h1 {
	font-family: Arial,Helvetica,Helv,sans-serif;
	font-size:	 25px;
}

@media screen {
	a:hover {
		color:	 			#fc6;
		text-decoration:	none;
	}
}

.toplevel {
	color:		 #306;
	font-family: Arial,Helvetica,Helv,sans-serif;
	font-size:	 14px;
	font-weight: bold;
}

.sublevel {
	font-family: Arial,Helvetica,Helv,sans-serif;
	font-size:	 12px;
	font-weight: bold;
}

.toponomic {
	font-family: Arial,Helvetica,Helv,sans-serif;
	font-size:	 12px;
	font-weight: bold;
}

.copyright {
	font-family: Arial,Helvetica,Helv,sans-serif;
	font-size:	 10px;
	font-style:  italic;
}
